/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.ui;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import benten.cat.ui.CatUiPlugin.CatUiPreference;
import benten.cat.ui.messages.CatPreferencePageMessages;
import benten.ui.views.UiToolkit;

/**
 * CAT プラグイン設定ページ。
 * 
 * @author KASHIHARA Shinji
 */
public class CatPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {
	/**
	 * CAT プラグイン設定ページのためのメッセージ。
	 */
	protected static final CatPreferencePageMessages fMsg = new CatPreferencePageMessages();

	/**
	 * Benten が提供する基本的なウィジェットを生成する UI ツールキットのインスタンス。
	 */
	private final UiToolkit toolkit = new UiToolkit();

	private Button showLineBreakCheck;

	@Override
	protected Control createContents(final Composite parent) {
		final Composite composite = toolkit.createComposite(parent, 1);

		final Group g = new Group(composite, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(1, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText(fMsg.getLabel1());
		final IPreferenceStore store = CatUiPlugin.getDefault().getPreferenceStore();

		showLineBreakCheck = new Button(g, SWT.CHECK);
		showLineBreakCheck.setText(fMsg.getLabel2());
		showLineBreakCheck.setSelection(store.getBoolean(CatUiPreference.SHOW_LINE_BREAK.name()));

		return composite;
	}

	/**
	 * {@inheritDoc}
	 */
	public void init(IWorkbench workbench) {
	}

	@Override
	public boolean performOk() {
		final IPreferenceStore store = CatUiPlugin.getDefault().getPreferenceStore();
		store.setValue(CatUiPreference.SHOW_LINE_BREAK.name(), showLineBreakCheck.getSelection());
		CatUiPlugin.getDefault().firePreferenceChange();
		return true;
	}

	@Override
	protected void performDefaults() {
		final IPreferenceStore store = CatUiPlugin.getDefault().getPreferenceStore();
		showLineBreakCheck.setSelection(store.getDefaultBoolean(CatUiPreference.SHOW_LINE_BREAK.name()));
	}

	/**
	 * 設定初期化クラス。
	 */
	public static class Initializer extends AbstractPreferenceInitializer {
		@Override
		public void initializeDefaultPreferences() {
			final IPreferenceStore store = CatUiPlugin.getDefault().getPreferenceStore();
			store.setDefault(CatUiPreference.SHOW_LINE_BREAK.name(), true);
		}
	}
}
